# flake8: noqa
from slackchat.tasks.channel import create_private_channel
from slackchat.tasks.user import update_users
from slackchat.tasks.webhook import (
    post_webhook,
    post_webhook_unpublish,
    post_webhook_republish,
    verify_webhook,
)



import re

from django.contrib.postgres.fields import JSONField
from django.db import models
from django.utils.safestring import mark_safe
from markdown import markdown

from .custom_content_template import CustomContentTemplate


class Message(models.Model):
    """
    A message posted in a Slack channel by a user.
    """

    timestamp = models.DateTimeField(unique=True)

    channel = models.ForeignKey(
        "Channel", related_name="messages", on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        "User", related_name="messages", on_delete=models.CASCADE
    )
    text = models.TextField()

    serialized = JSONField(blank=True, null=True)

    def text_summary(self):
        return self.text[:50]

    def html(self):
        return mark_safe(markdown(self.get_content()))

    def get_content(self):
        running_text = self.text
        matches = self.find_template_matches()

        if len(matches) > 0:
            for template, _ in matches:
                if template and template.content_template != "":
                    search = re.search(template.search_string, running_text)
                    if search:
                        groups = search.groups()
                        running_text = template.content_template.format(
                            *groups
                        )
        return running_text

    def get_custom_args(self):
        custom_args = []
        matches = self.find_template_matches()

        for template, match in matches:
            if match and template.argument_template:
                custom_args = [
                    str(arg).strip().format(*match.groups())
                    for arg in template.argument_template.split(",")
                ] + custom_args

        return custom_args

    def get_custom_kwargs(self):
        return self.parse_json_template("kwarg_template")

    def get_custom_attachment(self):
        return self.parse_json_template("attachment_template")

    def parse_json_template(self, template_type):
        output_dict = dict()

        matches = self.find_template_matches()
        if len(matches) > 0:
            for template_schema, match in matches:
                if match:
                    groups = match.groups()
                    template = getattr(template_schema, template_type)
                    if isinstance(template, dict):
                        for key in template:
                            template_key = getattr(
                                template_schema, template_type
                            )[key]
                            if isinstance(template_key, str):
                                output_dict[key] = template_key.format(*groups)
                            else:
                                output_dict[key] = template_key

        return output_dict

    def find_template_matches(self):
        matches = []

        for template in CustomContentTemplate.objects.filter(
            chat_type=self.channel.chat_type
        ):
            match = re.search(template.search_string, self.text)
            if match:
                matches.append((template, match))
        return matches

    def serialize(self):
        self.save()

    def save(self, *args, **kwargs):
        from slackchat.serializers import MessageSerializer

        self.serialized = MessageSerializer(self).data
        super().save(*args, **kwargs)

    def __str__(self):
        return self.text_summary()



from slackchat.models import Channel, ChatType
from slackchat.serializers import ChannelCMSSerializer, ChatTypeSerializer
from .base import CMSBase


class CMSList(CMSBase):
    template_name = "slackchat/list.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context["data"] = CMSBase.prep_data_for_injection(
            Channel.objects.order_by("publish_time", "title"),
            ChannelCMSSerializer,
            many=True,
        )

        context["all_chat_types"] = CMSBase.prep_data_for_injection(
            ChatType.objects.all(), ChatTypeSerializer, many=True
        )

        return context



import json
from django.views.generic import TemplateView
from django.urls import reverse_lazy
from django.core.serializers.json import DjangoJSONEncoder
from slackchat.conf import settings as app_settings
from slackchat.authentication import secure


@secure
class CMSBase(TemplateView):
    @staticmethod
    def prep_data_for_injection(queryset, serializer, many=False):
        if serializer:
            data = serializer(queryset, many=many).data
        else:
            data = {}

        return json.dumps(json.dumps(data, cls=DjangoJSONEncoder))

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["PUBLISH_ROOT"] = app_settings.PUBLISH_ROOT
        context["SLACK_TEAM_ROOT"] = app_settings.SLACK_TEAM_ROOT
        context["CMS_TOKEN"] = app_settings.CMS_TOKEN
        context["API_ROOT"] = reverse_lazy("cms-api")
        return context



from datetime import datetime

from django.core.exceptions import ObjectDoesNotExist
from markslack import MarkSlack
from slackchat.exceptions import MessageNotFoundError, ReactionNotFoundError
from slackchat.models import Argument, Channel, Message, Reaction, User

from slackchat.utils import log_event

marker = MarkSlack()


def strptimestamp(timestamp):
    return datetime.fromtimestamp(float(timestamp))


def handle_removed(id, event):
    item = event.get("item")

    if item.get("type", None) != "message":
        return

    item_user, created = User.objects.get_or_create(
        api_id=event.get("item_user")
    )

    user, created = User.objects.get_or_create(api_id=event.get("user"))

    try:
        channel = Channel.objects.get(api_id=item.get("channel"))
    except ObjectDoesNotExist:
        log_event(200, "REACTION_IGNORED")
        return

    try:
        message = Message.objects.get(
            timestamp=strptimestamp(item.get("ts")),
            channel=channel,
            user=item_user,
        )
    except ObjectDoesNotExist:
        raise MessageNotFoundError(
            "{} @ {} by {}".format(
                channel, strptimestamp(item.get("ts")), item_user
            )
        )

    try:
        reaction = Reaction.objects.get(
            message=message, user=user, reaction=event.get("reaction")
        )
    except ObjectDoesNotExist:
        raise ReactionNotFoundError("Can't find reaction.")

    log_event(200, "REACTION_DELETED")
    reaction.delete()


def handle_added(id, event):
    item = event.get("item")

    if item.get("type", None) != "message":
        return False

    item_user, created = User.objects.get_or_create(
        api_id=event.get("item_user")
    )

    try:
        channel = Channel.objects.get(api_id=item.get("channel"))
    except ObjectDoesNotExist:
        log_event(200, "REACTION_IGNORED")
        return

    try:
        message = Message.objects.get(
            timestamp=strptimestamp(item.get("ts")),
            channel=channel,
            user=item_user,
        )
    except ObjectDoesNotExist:
        raise MessageNotFoundError(
            "{} @ {} by {}".format(
                channel, strptimestamp(item.get("ts")), item_user
            )
        )

    reaction_user, created = User.objects.get_or_create(
        api_id=event.get("user")
    )

    reaction_kwargs = {
        "timestamp": strptimestamp(event.get("event_ts")),
        "message": message,
        "reaction": event.get("reaction"),
        "user": reaction_user,
    }

    try:
        argument = Argument.objects.get(
            character=event.get("reaction"), chat_type=channel.chat_type
        )
        reaction_kwargs["argument"] = argument
    except ObjectDoesNotExist:
        pass

    r, created = Reaction.objects.update_or_create(**reaction_kwargs)
    if created:
        log_event(200, "REACTION_ADDED")



# Generated by Django 2.0.4 on 2019-02-19 20:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0008_auto_20190219_1745'),
    ]

    operations = [
        migrations.AddField(
            model_name='channel',
            name='published',
            field=models.BooleanField(default=False, help_text='Determines if the page should be live (for renderer)'),
        ),
        migrations.AlterField(
            model_name='channel',
            name='publish_time',
            field=models.DateTimeField(blank=True, help_text='Dateline.', null=True),
        ),
    ]



from slackchat.utils import log_event

from slackchat.models import Attachment, Message


def handle(message_pk, attachment):
    # TODO: For now, we're not attaching unfurled tweets.
    # This gets handled by markslack.
    if attachment.get("service_name") == "twitter":
        return
    data = {
        "title": attachment.get("title"),
        "title_link": attachment.get("title_link"),
        "text": attachment.get("text"),
        "service_name": attachment.get("service_name"),
        "service_icon": attachment.get("service_icon"),
        "service_url": attachment.get("service_url"),
        "image_url": attachment.get("image_url"),
        "image_width": attachment.get("image_width"),
        "image_height": attachment.get("image_height"),
        "video_html": attachment.get("video_html"),
        "video_html_width": attachment.get("video_html_width"),
        "video_html_height": attachment.get("video_html_height"),
        "thumb_url": attachment.get("thumb_url"),
        "thumb_width": attachment.get("thumb_width"),
        "thumb_height": attachment.get("thumb_height"),
    }
    if data.get("title_link") or data.get("image_url"):
        message = Message.objects.get(pk=message_pk)
        a, created = Attachment.objects.get_or_create(message=message, **data)
        if created:
            log_event(200, "ATTACHMENT_CREATED")



import logging
from slackchat.conf import settings

slack_logger = logging.getLogger(settings.SLACK_LOGGER)


def log_event(code, note):
    slack_logger.info("SLACKCHAT EVENT        {}        {}".format(code, note))



from datetime import datetime
from django.utils.timezone import is_aware, make_aware

from django.core.exceptions import ObjectDoesNotExist
from markslack import MarkSlack

from slackchat.conf import settings
from slackchat.utils import log_event
from slackchat.exceptions import MessageNotFoundError, UserNotFoundError
from slackchat.models import Channel, KeywordArgument, Message, User


from .attachments import handle as handle_attachment

ignored_subtypes = ["group_join", "file_share", "group_archive"]


def strptimestamp(timestamp):
    date = datetime.fromtimestamp(float(timestamp))
    if not is_aware(date):
        date = make_aware(date)
    return date


def handle_removed(id, event):
    try:
        channel = Channel.objects.get(api_id=event.get("channel"))
    except ObjectDoesNotExist:
        log_event(200, "MESSAGE_IGNORED")
        return
    msg = {
        "user": event.get("previous_message").get("user"),
        "ts": event.get("previous_message").get("ts"),
        "text": event.get("message", {}).get("text", None),
    }
    user, created = User.objects.get_or_create(api_id=msg.get("user"))

    # If thread
    if not event.get("message", {}).get(
        "subtype"
    ) == "tombstone" and event.get("previous_message", {}).get("thread_ts"):
        if not channel.chat_type.kwargs_in_threads:
            return

        thread = event.get("previous_message")

        try:
            original_message = Message.objects.get(
                timestamp=strptimestamp(thread.get("thread_ts"))
            )
        except ObjectDoesNotExist:
            raise MessageNotFoundError(
                "{}".format(strptimestamp(thread.get("thread_ts")))
            )

        try:
            user = User.objects.get(api_id=thread.get("user"))
        except ObjectDoesNotExist:
            raise UserNotFoundError(thread.get("user"))

        try:
            kwarg = KeywordArgument.objects.get(
                timestamp=strptimestamp(thread.get("ts")),
                message=original_message,
                user=user,
            )
        except ObjectDoesNotExist:
            return

        log_event(200, "KWARG_DELETED")
        kwarg.delete()

    else:
        m = Message.objects.get(
            channel=channel, timestamp=strptimestamp(msg.get("ts")), user=user
        )
        log_event(200, "MESSAGE_DELETED")
        m.delete()


def handle(id, event):
    user_templates = {
        user.api_id: settings.MARKSLACK_USER_TEMPLATE(user)
        for user in User.objects.all()
    }

    marker = MarkSlack(
        user_templates=user_templates,
        link_templates=settings.MARKSLACK_LINK_TEMPLATES,
        image_template=settings.MARKSLACK_IMAGE_TEMPLATE,
    )

    try:
        channel = Channel.objects.get(api_id=event.get("channel"))
    except ObjectDoesNotExist:
        log_event(200, "MESSAGE_IGNORED")
        return

    subtype = event.get("subtype", None)
    if subtype in ignored_subtypes:
        return

    if subtype:
        msg = {
            "user": event.get("previous_message").get("user"),
            "ts": event.get("previous_message").get("ts"),
            "text": event.get("message", {}).get("text", None),
        }
    else:
        msg = {
            "user": event.get("user"),
            "ts": event.get("ts"),
            "text": event.get("text"),
        }

    user, created = User.objects.get_or_create(api_id=msg.get("user"))

    thread_ts = event.get("thread_ts", None) or event.get("message", {}).get(
        "thread_ts", None
    )
    # If thread
    if thread_ts and (
        event.get("parent_user_id", None)
        or event.get("message", {}).get("parent_user_id", None)
    ):
        if not channel.chat_type.kwargs_in_threads:
            return

        try:
            key, value = msg.get("text").split(": ", 1)
        except ValueError:
            # Don't handle threads that don't follow key-value pattern
            return

        try:
            original_message = Message.objects.get(
                timestamp=strptimestamp(thread_ts)
            )
        except ObjectDoesNotExist:
            raise MessageNotFoundError("{}".format(strptimestamp(thread_ts)))
        log_event(200, "KWARG_ADDED")
        KeywordArgument.objects.update_or_create(
            timestamp=strptimestamp(msg.get("ts")),
            message=original_message,
            user=user,
            defaults={"key": key, "value": value},
        )
    else:
        message, created = Message.objects.update_or_create(
            channel=channel,
            timestamp=strptimestamp(msg.get("ts")),
            user=user,
            defaults={"text": marker.mark(msg.get("text"))},
        )

        if created:
            log_event(200, "MESSAGE_CREATED")

        else:
            log_event(200, "MESSAGE_UPDATED")

        # Attachments
        attachments = event.get("message", {}).get("attachments")
        if attachments:
            for attachment in attachments:
                handle_attachment(message.pk, attachment)



import random
from django.conf import settings as project_settings


class Settings:
    pass


Settings.CMS_TOKEN = getattr(
    project_settings, "SLACKCHAT_CMS_TOKEN", "%032x" % random.getrandbits(128)
)

Settings.PUBLISH_ROOT = getattr(
    project_settings,
    "SLACKCHAT_PUBLISH_ROOT",
    "https://example.com/slackchat-root/",
)

Settings.SLACK_TEAM_ROOT = getattr(
    project_settings, "SLACKCHAT_TEAM_ROOT", "https://example.com/slack/"
)

Settings.MANAGERS = getattr(project_settings, "SLACKCHAT_MANAGERS", [])

Settings.DEFAULT_OWNER = getattr(
    project_settings, "SLACKCHAT_DEFAULT_OWNER", None
)

Settings.SLACK_VERIFICATION_TOKEN = getattr(
    project_settings, "SLACKCHAT_SLACK_VERIFICATION_TOKEN", None
)

Settings.SLACK_API_TOKEN = getattr(
    project_settings, "SLACKCHAT_SLACK_API_TOKEN", None
)

Settings.SLACK_LOGGER = getattr(
    project_settings, "SLACKCHAT_SLACK_LOGGER", "slackchat"
)

Settings.WEBHOOK_VERIFICATION_TOKEN = getattr(
    project_settings, "SLACK_WEBHOOK_VERIFICATION_TOKEN", "slackchat"
)

Settings.PUBLISH_ROOT = getattr(
    project_settings, "SLACKCHAT_PUBLISH_ROOT", None
)

Settings.MARKSLACK_USER_TEMPLATE = getattr(
    project_settings,
    "SLACK_MARKSLACK_USER_TEMPLATE",
    lambda user: '<span class="mention">{} {}</span>'.format(
        user.first_name, user.last_name
    ),
)

Settings.MARKSLACK_LINK_TEMPLATES = getattr(
    project_settings,
    "SLACK_MARKSLACK_LINK_TEMPLATES",
    {
        "twitter.com": '<blockquote class="twitter-tweet" data-lang="en"><a href="{}"></a></blockquote>'  # noqa
    },
)

Settings.MARKSLACK_IMAGE_TEMPLATE = getattr(
    project_settings, "SLACK_MARKSLACK_IMAGE_TEMPLATE", "[(image)]({})"
)


def default_user_image_upload_to(instance, filename):
    return "slackchat/users/{0}{1}/{2}".format(
        instance.first_name, instance.last_name, filename
    )


Settings.USER_IMAGE_UPLOAD_TO = getattr(
    project_settings,
    "SLACKCHAT_USER_IMAGE_UPLOAD_TO",
    default_user_image_upload_to,
)


settings = Settings



import os

from setuptools import find_packages, setup

import slackchat

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name=slackchat.__package__,
    version=slackchat.__version__,
    packages=find_packages(exclude=("example", "docs")),
    include_package_data=True,
    url="https://github.com/The-Politico/django-slackchat-serializer",
    license="MIT",
    description="A Django app that serializes conversations in Slack.",
    python_requires=">=3",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    install_requires=[
        "celery",
        "djangorestframework",
        "django-foreignform",
        "emoji",
        "markdown",
        "markslack",
        "Pillow",
        "slackclient>=1,<2",
        "slacker",
        "tqdm",
    ],
)



# Generated by Django 2.0.4 on 2019-02-19 17:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0007_message_serialized'),
    ]

    operations = [
        migrations.AlterField(
            model_name='channel',
            name='publish_time',
            field=models.DateTimeField(help_text='Dateline.', null=True),
        ),
    ]



import json
from django.contrib import admin
from foreignform.mixins import ForeignFormAdminMixin

from .serializers import ChannelSerializer

from .celery import (
    post_webhook,
    post_webhook_republish,
    post_webhook_unpublish,
    update_users,
)
from .models import (
    Argument,
    Attachment,
    Channel,
    ChatType,
    CustomContentTemplate,
    KeywordArgument,
    Message,
    Reaction,
    User,
    Webhook,
)


class ChatTypeAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {"fields": ("name", "publish_path")}),
        (
            "Channel options",
            {"fields": ("render_to_html", "kwargs_in_threads")},
        ),
        (
            "Extra channel fields",
            {
                "fields": ("jsonSchema", "uiSchema"),
                "description": (
                    'See the documentation for <a href="'
                    'https://github.com/mozilla-services/react-jsonschema-form"'
                    ' target="_blank"> '
                    "react-jsonschema-form</a> to learn how to configure extra "
                    "fields for channels of this type."
                ),
            },
        ),
    )


class ChannelAdmin(ForeignFormAdminMixin, admin.ModelAdmin):
    foreignform_foreign_key = "chat_type"
    foreignform_field = "extras"
    readonly_fields = ("api_id", "team_id")
    fieldsets = (
        (None, {"fields": ("api_id", "team_id", "chat_type", "owner")}),
        (
            "Publishing info",
            {"fields": ("publish_path", "publish_time", "live")},
        ),
        ("Page content", {"fields": ("title", "introduction")}),
        (
            "SEO tags",
            {"fields": ("meta_title", "meta_description", "meta_image")},
        ),
        ("Extra channel fields", {"fields": ("extras",)}),
    )
    list_display = (
        "slackchat",
        "published_link",
        "api_link",
        "slack_link",
        "live",
    )
    autocomplete_fields = ["owner"]
    actions = [
        "request_chats_update",
        "request_chats_republish",
        "request_chats_unpublish",
        "close_live_chats",
    ]

    def request_chats_update(self, request, queryset):
        for channel in queryset:
            post_webhook.delay(channel.id.hex, channel.chat_type.name)
        self.message_user(request, "Requested channels update chat page!")

    def request_chats_republish(self, request, queryset):
        for channel in queryset:
            post_webhook_republish.delay(
                ChannelSerializer(channel).data, channel.chat_type.name
            )
        self.message_user(request, "Requested channels republish chat page!")

    def request_chats_unpublish(self, request, queryset):
        for channel in queryset:
            post_webhook_unpublish.delay(
                ChannelSerializer(channel).data, channel.chat_type.name
            )
        self.message_user(request, "Requested channels unpublish chat page!")

    def close_live_chats(self, request, queryset):
        queryset.update(live=False)
        for channel in queryset:
            post_webhook.delay(channel.id.hex, channel.chat_type.name)
        self.message_user(
            request,
            "Closed live channels and requested they update chat page!",
        )


class UserAdmin(admin.ModelAdmin):
    search_fields = ["first_name", "last_name"]
    actions = ["update_user_profiles"]

    def update_user_profiles(self, request, queryset):
        update_users.delay([user.pk for user in queryset])
        self.message_user(request, "Updated user profiles!")


class WebhookAdmin(admin.ModelAdmin):
    fields = ("endpoint", "verified")


class MessageAdmin(admin.ModelAdmin):
    actions = ["serialize"]
    list_display = ("timestamp", "channel", "user", "text_summary")

    def serialize(self, request, queryset):
        for msg in queryset:
            msg.serialize()
        self.message_user(request, "Messages have been reserialized")


admin.site.register(Argument)
admin.site.register(Attachment)
admin.site.register(Channel, ChannelAdmin)
admin.site.register(User, UserAdmin)
admin.site.register(CustomContentTemplate)
admin.site.register(Message, MessageAdmin)
admin.site.register(Reaction)
admin.site.register(KeywordArgument)
admin.site.register(ChatType, ChatTypeAdmin)
admin.site.register(Webhook, WebhookAdmin)



from importlib import import_module

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.utils.decorators import method_decorator
from rest_framework import authentication, exceptions
from slackchat.conf import settings as app_settings


class TokenAPIAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        # Don't enforce if DEBUG
        if False:
            return (AnonymousUser, None)
        try:
            # Per DRF token auth, token is prefixed by string
            # literal "Token" plus whitespace, e.g., "Token <AUTHTOKEN>"
            token = request.META.get('HTTP_AUTHORIZATION').split()[1]
        except:
            raise exceptions.AuthenticationFailed(
                'No token or incorrect token format')

        if token == getattr(app_settings, 'CMS_TOKEN'):
            return (AnonymousUser, None)
        raise exceptions.AuthenticationFailed('Unauthorized')


def import_class(val):
    """Attempt to import a class from a string representation.
    Pattern borrowed from Django REST Framework.
    See rest_framework/settings.py#L170-L182
    """
    try:
        parts = val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = import_module(module_path)
        return getattr(module, class_name)
    except (ImportError, AttributeError) as e:
        msg = "Could not import auth/permission class '{}'. {}: {}.".format(
            val,
            e.__class__.__name__,
            e)
        raise ImportError(msg)


def secure(view):
    """Set an auth decorator applied for views.
    If DEBUG is on, we serve the view without authenticating.
    Default is 'django.contrib.auth.decorators.login_required'.
    Can also be 'django.contrib.admin.views.decorators.staff_member_required'
    or a custom decorator.
    """
    AUTH = getattr(
        settings,
        'SLACKCHAT_AUTH_DECORATOR',
        'django.contrib.admin.views.decorators.staff_member_required'
    )
    auth_decorator = import_class(AUTH)
    return method_decorator(auth_decorator, name='dispatch')(view)
    # return (
    #     view if settings.DEBUG
    #     else method_decorator(auth_decorator, name='dispatch')(view)
    # )
